//
//  TVSpotlightManager.m
//  TeeVee2
//
//  Created by Tomas Perzl on 26.09.15.
//  Copyright © 2015 Lukáš Tesař. All rights reserved.
//

#import "TVSpotlightManager.h"
#import <CoreSpotlight/CoreSpotlight.h>
#import <MobileCoreServices/MobileCoreServices.h>
#import "NSDate-Utilities.h"

@implementation TVSpotlightManager

+(id)sharedManager{
    static TVSpotlightManager *sharedMyManager = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedMyManager = [[TVSpotlightManager alloc] init];
    });
    return sharedMyManager;
}

-(void)addSearchableShow:(TVShow *)show{
    
    NSOperatingSystemVersion ios9_0 = (NSOperatingSystemVersion){9, 0};
    if ([[NSProcessInfo processInfo] isOperatingSystemAtLeastVersion:ios9_0]) {

        if (!show.name || !show.idTrakt) {
            return ;
        }

        CSSearchableItemAttributeSet *attributeSet = [[CSSearchableItemAttributeSet alloc]initWithItemContentType:(NSString *)kUTTypeMovie];
    
        attributeSet.title = show.visibleName;
        NSString *description = @"";
        NSCalendar *calendar = [NSCalendar currentCalendar];
        NSDate *date = [NSDate date];
    
        if (show.upcomingEpisode) {
            NSArray *nextEpisode = [NSArray new];
            if ([show.upcomingEpisode.visibleAirDate isTodayWithCal:calendar]) {
                nextEpisode = [NSArray arrayWithObjects:[NSString stringWithFormat:@"Next up: %@ TODAY",show.upcomingEpisode.visibleIndex], show.upcomingEpisode.name, nil];
            }else if ([show.upcomingEpisode.visibleAirDate isTomorrowWithCal:calendar]){
                nextEpisode = [NSArray arrayWithObjects:[NSString stringWithFormat:@"Next up: %@ TOMORROW",show.upcomingEpisode.visibleIndex], show.upcomingEpisode.name, nil];
            }else if(show.upcomingEpisode.visibleAirDate){
                nextEpisode = [NSArray arrayWithObjects:[NSString stringWithFormat:@"Next up: %@ in %ld days",show.upcomingEpisode.visibleIndex,(long)[[self class] daysWithinEraFromDate:date toDate:show.upcomingEpisode.visibleAirDate usingCalendar:calendar]], show.upcomingEpisode.name, nil];
            }else{
                nextEpisode = [NSArray arrayWithObjects:[NSString stringWithFormat:@"Next up: %@ TBA",show.upcomingEpisode.visibleIndex], show.upcomingEpisode.name, nil];
            }
            
            for (NSString *string in nextEpisode) {
                if (string.length != 0 && string != nil) {
                    description = [description.length == 0 ? description : [description stringByAppendingString:@"\r"] stringByAppendingString: string];
                }
            }
        }else if (!show.upcomingEpisode){
            description = show.summary;
        }
    
        attributeSet.contentDescription = description;
        attributeSet.contentCreationDate = show.startDate;
    
        NSString *characters = @"";
        for (TVCharacter *character in show.characters) {
            if (character.name.length != 0 && character.name != nil) {
                characters = [characters.length == 0 ? characters : [characters stringByAppendingString:@" "] stringByAppendingString:character.name];
            }
        }

        NSString *showAndCharacters = [[characters stringByAppendingString:@" "] stringByAppendingString: (show.visibleName != nil) ? show.visibleName : show.name];
        NSArray *keyWords = [showAndCharacters componentsSeparatedByString:@" "];
     
        NSString *genres = @"";
        for (TVGenre *genre in show.genres) {
            if (genre.name.length != 0 && genre.name != nil) {
                genres = [genres.length == 0 ? genres : [genres stringByAppendingString:@" "] stringByAppendingString:genre.name];
            }
        }
    
        attributeSet.keywords = keyWords;
    
        [show.imagePoster loadImageWithCompletion:nil];
    
        NSData *imageData = [NSData dataWithData:UIImagePNGRepresentation(show.imagePoster.image)];
        attributeSet.thumbnailData = imageData;
    
        float rating = ([show.rating floatValue] / 2);
        NSNumber *showRating = [NSNumber numberWithFloat:rating];
        attributeSet.rating = showRating;
        attributeSet.genre = genres;
    
        CSSearchableItem *item = [[CSSearchableItem alloc]initWithUniqueIdentifier:[NSString stringWithFormat:@"teevee.%@",show.idTrakt] domainIdentifier:@"com.crazyapps" attributeSet:attributeSet];
    
        [[CSSearchableIndex defaultSearchableIndex] indexSearchableItems:@[item] completionHandler: ^(NSError * __nullable error) {
            if (error) {
                NSLog(@"Error! :%@",error);
            }
        }];
    }
    
}

-(void)removeSearchableShow:(TVShow *)show{
    
    NSOperatingSystemVersion ios9_0 = (NSOperatingSystemVersion){9, 0};
    if ([[NSProcessInfo processInfo] isOperatingSystemAtLeastVersion:ios9_0]) {
        NSString *identifierToDelete = [NSString stringWithFormat:@"teevee.%@",show.idTrakt];
        [[CSSearchableIndex defaultSearchableIndex]
         deleteSearchableItemsWithIdentifiers:@[identifierToDelete] completionHandler:^(NSError *deletionError) {
             if (deletionError)
             {
                 NSLog(@"Could not delete items from the search index with error %@", deletionError);
             }
         }];
    }
}

-(void)updateSearchableShow:(TVShow *)show{ //I think that Item cannot be updated, so remove the item before and add it again.
    NSOperatingSystemVersion ios9_0 = (NSOperatingSystemVersion){9, 0};
    if ([[NSProcessInfo processInfo] isOperatingSystemAtLeastVersion:ios9_0]) {
        [self removeSearchableShow:show];
        [self addSearchableShow:show];
    }
}

+ (NSInteger)daysWithinEraFromDate:(NSDate *)startDate
                            toDate:(NSDate *)endDate
                     usingCalendar:(NSCalendar *)cal
{
    NSTimeZone *localTimeZone   = [NSTimeZone localTimeZone];
    NSInteger secondsFromGMT    = [localTimeZone secondsFromGMT];
    
    [cal setTimeZone:localTimeZone];
    NSDate *newDate1 = [startDate dateByAddingTimeInterval:secondsFromGMT];
    NSDate *newDate2 = [endDate dateByAddingTimeInterval:secondsFromGMT];
    
    NSInteger startDay  = [cal ordinalityOfUnit:NSCalendarUnitDay
                                         inUnit:NSCalendarUnitEra
                                        forDate:newDate1];
    NSInteger endDay    = [cal ordinalityOfUnit:NSCalendarUnitDay
                                         inUnit:NSCalendarUnitEra
                                        forDate:newDate2];
    return (endDay - startDay);
}


@end
